@@ -63,7 +63,7 @@ PROGRESSDISPLAY::~PROGRESSDISPLAY (void)
 
 void PROGRESSDISPLAY::Init (HWND hWnd)
 {
-   SetWindowLongPtr (hWnd, DWLP_USER, (LONG)this);
+   SetWindowLongPtr (hWnd, DWLP_USER, (LONG_PTR)this);
    Subclass_AddHook (hWnd, PROGRESSDISPLAY::ProgressDisplay_HookProc);
 
    m_msgFinish = 0;
@@ -176,13 +176,13 @@ LPPROGRESSDISPLAY PROGRESSDISPLAY::GetProgressDisplay (HWND hWnd)
 }
 
 
-BOOL CALLBACK PROGRESSDISPLAY::ProgressDisplay_StubProc (HWND hWnd, UINT msg, WPARAM wp, LPARAM lp)
+HRESULT CALLBACK PROGRESSDISPLAY::ProgressDisplay_StubProc (HWND hWnd, UINT msg, WPARAM wp, LPARAM lp)
 {
    return FALSE;
 }
 
 
-BOOL CALLBACK PROGRESSDISPLAY::ProgressDisplay_HookProc (HWND hWnd, UINT msg, WPARAM wp, LPARAM lp)
+HRESULT CALLBACK PROGRESSDISPLAY::ProgressDisplay_HookProc (HWND hWnd, UINT msg, WPARAM wp, LPARAM lp)
 {
    PVOID oldproc = Subclass_FindNextHook (hWnd, PROGRESSDISPLAY::ProgressDisplay_HookProc);
 
