@@ -16,20 +16,17 @@
 #include <string.h>
 #include <malloc.h>
 #include <osi.h>
-#include <rx/rx.h>
-#ifndef DJGPP
-#include <rx/rxkad.h>
-#else
-#include <rx/rxkad.h>
-#endif
-
 #include "afsd.h"
+#include <rx/rx.h>
+#include <rx/rxkad.h>
+#include <afs/unified_afs.h>
+#include <WINNT/afsreg.h>
 
 osi_rwlock_t cm_connLock;
 
-long RDRtimeout = CM_CONN_DEFAULTRDRTIMEOUT;
-long ConnDeadtimeout = CM_CONN_CONNDEADTIME;
-long HardDeadtimeout = CM_CONN_HARDDEADTIME;
+DWORD RDRtimeout = CM_CONN_DEFAULTRDRTIMEOUT;
+unsigned short ConnDeadtimeout = CM_CONN_CONNDEADTIME;
+unsigned short HardDeadtimeout = CM_CONN_HARDDEADTIME;
 
 #define LANMAN_WKS_PARAM_KEY "SYSTEM\\CurrentControlSet\\Services\\lanmanworkstation\\parameters"
 #define LANMAN_WKS_SESSION_TIMEOUT "SessTimeout"
@@ -45,10 +42,11 @@ void cm_PutConn(cm_conn_t *connp)
 
 void cm_InitConn(void)
 {
-	static osi_once_t once;
-	long code;
-	DWORD sessTimeout;
-	HKEY parmKey;
+    static osi_once_t once;
+    long code;
+    DWORD dwValue;
+    DWORD dummyLen;
+    HKEY parmKey;
         
     if (osi_Once(&once)) {
 	lock_InitializeRWLock(&cm_connLock, "connection global lock");
@@ -64,25 +62,40 @@ void cm_InitConn(void)
 			    0, KEY_QUERY_VALUE, &parmKey);
 	if (code == ERROR_SUCCESS)
         {
-	    DWORD dummyLen = sizeof(sessTimeout);
+	    dummyLen = sizeof(DWORD);
 	    code = RegQueryValueEx(parmKey, LANMAN_WKS_SESSION_TIMEOUT, NULL, NULL, 
-				   (BYTE *) &sessTimeout, &dummyLen);
+				   (BYTE *) &dwValue, &dummyLen);
 	    if (code == ERROR_SUCCESS)
-            {
-                RDRtimeout = sessTimeout;
-	    }
+                RDRtimeout = dwValue;
+	    RegCloseKey(parmKey);
         }
 
-	afsi_log("lanmanworkstation : SessTimeout %d", sessTimeout);
-	if ( ConnDeadtimeout < RDRtimeout + 15 ) {
-	    ConnDeadtimeout = RDRtimeout + 15;
-	    afsi_log("ConnDeadTimeout increased to %d", ConnDeadtimeout);
-	}
-	if ( HardDeadtimeout < 2 * ConnDeadtimeout ) {
-	    HardDeadtimeout = 2 * ConnDeadtimeout;
-	    afsi_log("HardDeadTimeout increased to %d", HardDeadtimeout);
+	code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
+			     0, KEY_QUERY_VALUE, &parmKey);
+	if (code == ERROR_SUCCESS) {
+	    dummyLen = sizeof(DWORD);
+	    code = RegQueryValueEx(parmKey, "ConnDeadTimeout", NULL, NULL,
+				    (BYTE *) &dwValue, &dummyLen);
+	    if (code == ERROR_SUCCESS)
+                ConnDeadtimeout = (unsigned short)dwValue;
+
+	    dummyLen = sizeof(DWORD);
+	    code = RegQueryValueEx(parmKey, "HardDeadTimeout", NULL, NULL,
+				    (BYTE *) &dwValue, &dummyLen);
+	    if (code == ERROR_SUCCESS)
+                HardDeadtimeout = (unsigned short)dwValue;
+	    afsi_log("HardDeadTimeout is %d", HardDeadtimeout);
+	    RegCloseKey(parmKey);
 	}
 
+	afsi_log("lanmanworkstation : SessTimeout %u", RDRtimeout);
+	if (ConnDeadtimeout == 0)
+	    ConnDeadtimeout = (unsigned short) (RDRtimeout / 2);
+	afsi_log("ConnDeadTimeout is %d", ConnDeadtimeout);
+	if (HardDeadtimeout == 0)
+	    HardDeadtimeout = (unsigned short) RDRtimeout;
+	afsi_log("HardDeadTimeout is %d", HardDeadtimeout);
+
 	osi_EndOnce(&once);
     }
 }
@@ -154,9 +167,10 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     int dead_session;
     long timeUsed, timeLeft;
     long code;
-        
-    osi_Log2(afsd_logp, "cm_Analyze connp 0x%x, code 0x%x",
-             (long) connp, errorCode);
+    char addr[16];
+
+    osi_Log2(afsd_logp, "cm_Analyze connp 0x%p, code 0x%x",
+             connp, errorCode);
 
     /* no locking required, since connp->serverp never changes after
      * creation */
@@ -337,31 +351,46 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     else if (errorCode == VNOVOL || errorCode == VMOVED || errorCode == VOFFLINE ||
              errorCode == VSALVAGE || errorCode == VNOSERVICE || errorCode == VIO) 
     {       
-#ifndef DJGPP
-        HANDLE h;
-        char *ptbuf[1];
-        char s[100];
-#endif
         char addr[16];
         char *format;
-
+#ifndef DJGPP
+	DWORD msgID;
+#endif
         switch ( errorCode ) {
         case VNOVOL:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VNOVOL;
+#endif
             format = "Server %s reported volume %d as not attached.";
             break;
         case VMOVED:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VMOVED;
+#endif
             format = "Server %s reported volume %d as moved.";
             break;
         case VOFFLINE:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VOFFLINE;
+#endif
             format = "Server %s reported volume %d as offline.";
             break;
         case VSALVAGE:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VSALVAGE;
+#endif
             format = "Server %s reported volume %d as needs salvage.";
             break;
         case VNOSERVICE:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VNOSERVICE;
+#endif
             format = "Server %s reported volume %d as not in service.";
             break;
         case VIO:
+#ifndef DJGPP
+	    msgID = MSG_SERVER_REPORTS_VIO;
+#endif
             format = "Server %s reported volume %d as temporarily unaccessible.";
             break;
         }
@@ -372,16 +401,10 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
                  ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
                  ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
                  ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24)); 
-        osi_Log2(afsd_logp, format, osi_LogSaveString(afsd_logp,addr), fidp->volume);
-        
+
+	osi_Log2(afsd_logp, format, osi_LogSaveString(afsd_logp,addr), fidp->volume);
 #ifndef DJGPP
-        /* Create Event Log message */
-        h = RegisterEventSource(NULL, AFS_DAEMON_EVENT_NAME);
-        sprintf(s, format, addr, fidp->volume);
-        ptbuf[0] = s;
-        ReportEvent(h, EVENTLOG_WARNING_TYPE, 0, 1009, NULL,
-                     1, 0, ptbuf, NULL);
-        DeregisterEventSource(h);
+	LogEvent(EVENTLOG_WARNING_TYPE, msgID, addr, fidp->volume);
 #endif
 
         /* Mark server offline for this volume */
@@ -411,23 +434,27 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 	    scp = cm_FindSCache(fidp);
 	    if (scp) {
 		cm_scache_t *pscp = NULL;
-		
+
 		if (scp->fileType != CM_SCACHETYPE_DIRECTORY)
 		    pscp = cm_FindSCacheParent(scp);
 
-		lock_ObtainWrite(&cm_scacheLock);
-		cm_RecycleSCache(scp, CM_SCACHE_RECYCLEFLAG_DESTROY_BUFFERS);
-		cm_ReleaseSCacheNoLock(scp);
-		lock_ReleaseWrite(&cm_scacheLock);
 
-		if (pscp) {
-		    if (pscp->cbExpires > 0 && pscp->cbServerp != NULL) {
-			lock_ObtainMutex(&pscp->mx);
-			cm_DiscardSCache(pscp);
-			lock_ReleaseMutex(&pscp->mx);
-		    }
-		    cm_ReleaseSCache(pscp);
-		}
+		lock_ObtainMutex(&scp->mx);
+		lock_ObtainWrite(&cm_scacheLock);
+		cm_RemoveSCacheFromHashTable(scp);
+		lock_ReleaseWrite(&cm_scacheLock);
+		scp->flags |= CM_SCACHEFLAG_DELETED;
+		lock_ReleaseMutex(&scp->mx);
+		cm_ReleaseSCache(scp);
+
+ 		if (pscp) {
+		    if (cm_HaveCallback(pscp)) {
+ 			lock_ObtainMutex(&pscp->mx);
+ 			cm_DiscardSCache(pscp);
+ 			lock_ReleaseMutex(&pscp->mx);
+ 		    }
+ 		    cm_ReleaseSCache(pscp);
+ 		}
 	    }
 	} else {
 	    osi_Log0(afsd_logp, "cm_Analyze passed VNOVNODE unknown fid.");
@@ -441,20 +468,21 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
          * this is to prevent the SMB session from timing out
          * In addition, we log an event to the event log 
          */
+
+        /* Log server being offline for this volume */
+        sprintf(addr, "%d.%d.%d.%d", 
+                 ((serverp->addr.sin_addr.s_addr & 0xff)),
+                 ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
+                 ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
+                 ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24)); 
+
 #ifndef DJGPP
-        HANDLE h;
-        char *ptbuf[1];
-        char s[100];
-        h = RegisterEventSource(NULL, AFS_DAEMON_EVENT_NAME);
-        sprintf(s, "cm_Analyze: HardDeadTime exceeded.");
-        ptbuf[0] = s;
-        ReportEvent(h, EVENTLOG_WARNING_TYPE, 0, 1009, NULL,
-                     1, 0, ptbuf, NULL);
-        DeregisterEventSource(h);
+	LogEvent(EVENTLOG_WARNING_TYPE, MSG_RX_HARD_DEAD_TIME_EXCEEDED, addr);
 #endif /* !DJGPP */
 	  
         retry = 0;
-        osi_Log0(afsd_logp, "cm_Analyze: hardDeadTime exceeded");
+        osi_Log1(afsd_logp, "cm_Analyze: hardDeadTime exceeded addr[%s]",
+		 osi_LogSaveString(afsd_logp,addr));
     }
     else if (errorCode >= -64 && errorCode < 0) {
         /* mark server as down */
@@ -516,59 +544,66 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
             case VRESTARTING       : s = "VRESTARTING";        break;
             case VREADONLY         : s = "VREADONLY";          break;
             case EAGAIN            : s = "EAGAIN";             break;
+	    case UAEAGAIN          : s = "UAEAGAIN";	       break;
+            case EINVAL            : s = "EINVAL";             break;
+	    case UAEINVAL          : s = "UAEINVAL";	       break;
             case EACCES            : s = "EACCES";             break;
-	    case ENOENT            : s = "ENOENT";             break;
-            case CM_ERROR_NOSUCHCELL	    : s = "CM_ERROR_NOSUCHCELL";         break; 			
-            case CM_ERROR_NOSUCHVOLUME	    : s = "CM_ERROR_NOSUCHVOLUME";       break; 			
-            case CM_ERROR_TIMEDOUT	    : s = "CM_ERROR_TIMEDOUT";           break; 		
-            case CM_ERROR_RETRY		    : s = "CM_ERROR_RETRY";              break; 
-            case CM_ERROR_NOACCESS	    : s = "CM_ERROR_NOACCESS";           break; 
-            case CM_ERROR_NOSUCHFILE	    : s = "CM_ERROR_NOSUCHFILE";         break; 			
-            case CM_ERROR_STOPNOW	    : s = "CM_ERROR_STOPNOW";            break; 			
-            case CM_ERROR_TOOBIG	    : s = "CM_ERROR_TOOBIG";             break; 				
-            case CM_ERROR_INVAL		    : s = "CM_ERROR_INVAL";              break; 				
-            case CM_ERROR_BADFD		    : s = "CM_ERROR_BADFD";              break; 				
-            case CM_ERROR_BADFDOP	    : s = "CM_ERROR_BADFDOP";            break; 			
-            case CM_ERROR_EXISTS	    : s = "CM_ERROR_EXISTS";             break; 				
-            case CM_ERROR_CROSSDEVLINK	    : s = "CM_ERROR_CROSSDEVLINK";       break; 			
-            case CM_ERROR_BADOP		    : s = "CM_ERROR_BADOP";              break; 				
-            case CM_ERROR_BADPASSWORD       : s = "CM_ERROR_BADPASSWORD";        break;         
-            case CM_ERROR_NOTDIR	    : s = "CM_ERROR_NOTDIR";             break; 				
-            case CM_ERROR_ISDIR		    : s = "CM_ERROR_ISDIR";              break; 				
-            case CM_ERROR_READONLY	    : s = "CM_ERROR_READONLY";           break; 			
-            case CM_ERROR_WOULDBLOCK	    : s = "CM_ERROR_WOULDBLOCK";         break; 			
-            case CM_ERROR_QUOTA		    : s = "CM_ERROR_QUOTA";              break; 				
-            case CM_ERROR_SPACE		    : s = "CM_ERROR_SPACE";              break; 				
-            case CM_ERROR_BADSHARENAME	    : s = "CM_ERROR_BADSHARENAME";       break; 			
-            case CM_ERROR_BADTID	    : s = "CM_ERROR_BADTID";             break; 				
-            case CM_ERROR_UNKNOWN	    : s = "CM_ERROR_UNKNOWN";            break; 			
-            case CM_ERROR_NOMORETOKENS	    : s = "CM_ERROR_NOMORETOKENS";       break; 			
-            case CM_ERROR_NOTEMPTY	    : s = "CM_ERROR_NOTEMPTY";           break; 			
-            case CM_ERROR_USESTD	    : s = "CM_ERROR_USESTD";             break; 				
-            case CM_ERROR_REMOTECONN	    : s = "CM_ERROR_REMOTECONN";         break; 			
-            case CM_ERROR_ATSYS		    : s = "CM_ERROR_ATSYS";              break; 				
-            case CM_ERROR_NOSUCHPATH	    : s = "CM_ERROR_NOSUCHPATH";         break; 			
-            case CM_ERROR_CLOCKSKEW	    : s = "CM_ERROR_CLOCKSKEW";          break; 			
-            case CM_ERROR_BADSMB	    : s = "CM_ERROR_BADSMB";             break; 				
-            case CM_ERROR_ALLBUSY	    : s = "CM_ERROR_ALLBUSY";            break; 			
-            case CM_ERROR_NOFILES	    : s = "CM_ERROR_NOFILES";            break; 			
-            case CM_ERROR_PARTIALWRITE	    : s = "CM_ERROR_PARTIALWRITE";       break; 			
-            case CM_ERROR_NOIPC		    : s = "CM_ERROR_NOIPC";              break; 				
-            case CM_ERROR_BADNTFILENAME	    : s = "CM_ERROR_BADNTFILENAME";      break; 			
-            case CM_ERROR_BUFFERTOOSMALL    : s = "CM_ERROR_BUFFERTOOSMALL";     break; 			
-            case CM_ERROR_RENAME_IDENTICAL  : s = "CM_ERROR_RENAME_IDENTICAL";   break; 		
-            case CM_ERROR_ALLOFFLINE        : s = "CM_ERROR_ALLOFFLINE";         break;          
-            case CM_ERROR_AMBIGUOUS_FILENAME: s = "CM_ERROR_AMBIGUOUS_FILENAME"; break;  
-            case CM_ERROR_BADLOGONTYPE	    : s = "CM_ERROR_BADLOGONTYPE";       break; 	    
-            case CM_ERROR_GSSCONTINUE       : s = "CM_ERROR_GSSCONTINUE";        break;         
-            case CM_ERROR_TIDIPC            : s = "CM_ERROR_TIDIPC";             break;              
-            case CM_ERROR_TOO_MANY_SYMLINKS : s = "CM_ERROR_TOO_MANY_SYMLINKS";  break;   
-            case CM_ERROR_PATH_NOT_COVERED  : s = "CM_ERROR_PATH_NOT_COVERED";   break;    
-            case CM_ERROR_LOCK_CONFLICT     : s = "CM_ERROR_LOCK_CONFLICT";      break;       
-            case CM_ERROR_SHARING_VIOLATION : s = "CM_ERROR_SHARING_VIOLATION";  break;   
-            case CM_ERROR_ALLDOWN           : s = "CM_ERROR_ALLDOWN";            break;             
-            case CM_ERROR_TOOFEWBUFS	    : s = "CM_ERROR_TOOFEWBUFS";         break; 			
-            case CM_ERROR_TOOMANYBUFS	    : s = "CM_ERROR_TOOMANYBUFS";        break; 			
+	    case UAEACCES 	   : s = "UAEACCES";           break;
+	    case ENOENT            : s = "ENOENT"; 	       break;
+	    case UAENOENT          : s = "UAENOENT";           break;
+	    case VICECONNBAD	   : s = "VICECONNBAD";	       break;
+	    case VICETOKENDEAD     : s = "VICETOKENDEAD";      break;
+	    case CM_ERROR_NOSUCHCELL	    : s = "CM_ERROR_NOSUCHCELL";         break; 			
+	    case CM_ERROR_NOSUCHVOLUME	    : s = "CM_ERROR_NOSUCHVOLUME";       break; 			
+	    case CM_ERROR_TIMEDOUT	    : s = "CM_ERROR_TIMEDOUT";           break; 		
+	    case CM_ERROR_RETRY		    : s = "CM_ERROR_RETRY";              break; 
+	    case CM_ERROR_NOACCESS	    : s = "CM_ERROR_NOACCESS";           break; 
+	    case CM_ERROR_NOSUCHFILE	    : s = "CM_ERROR_NOSUCHFILE";         break; 			
+	    case CM_ERROR_STOPNOW	    : s = "CM_ERROR_STOPNOW";            break; 			
+	    case CM_ERROR_TOOBIG	    : s = "CM_ERROR_TOOBIG";             break; 				
+	    case CM_ERROR_INVAL		    : s = "CM_ERROR_INVAL";              break; 				
+	    case CM_ERROR_BADFD		    : s = "CM_ERROR_BADFD";              break; 				
+	    case CM_ERROR_BADFDOP	    : s = "CM_ERROR_BADFDOP";            break; 			
+	    case CM_ERROR_EXISTS	    : s = "CM_ERROR_EXISTS";             break; 				
+	    case CM_ERROR_CROSSDEVLINK	    : s = "CM_ERROR_CROSSDEVLINK";       break; 			
+	    case CM_ERROR_BADOP		    : s = "CM_ERROR_BADOP";              break; 				
+	    case CM_ERROR_BADPASSWORD       : s = "CM_ERROR_BADPASSWORD";        break;         
+	    case CM_ERROR_NOTDIR	    : s = "CM_ERROR_NOTDIR";             break; 				
+	    case CM_ERROR_ISDIR		    : s = "CM_ERROR_ISDIR";              break; 				
+	    case CM_ERROR_READONLY	    : s = "CM_ERROR_READONLY";           break; 			
+	    case CM_ERROR_WOULDBLOCK	    : s = "CM_ERROR_WOULDBLOCK";         break; 			
+	    case CM_ERROR_QUOTA		    : s = "CM_ERROR_QUOTA";              break; 				
+	    case CM_ERROR_SPACE		    : s = "CM_ERROR_SPACE";              break; 				
+	    case CM_ERROR_BADSHARENAME	    : s = "CM_ERROR_BADSHARENAME";       break; 			
+	    case CM_ERROR_BADTID	    : s = "CM_ERROR_BADTID";             break; 				
+	    case CM_ERROR_UNKNOWN	    : s = "CM_ERROR_UNKNOWN";            break; 			
+	    case CM_ERROR_NOMORETOKENS	    : s = "CM_ERROR_NOMORETOKENS";       break; 			
+	    case CM_ERROR_NOTEMPTY	    : s = "CM_ERROR_NOTEMPTY";           break; 			
+	    case CM_ERROR_USESTD	    : s = "CM_ERROR_USESTD";             break; 				
+	    case CM_ERROR_REMOTECONN	    : s = "CM_ERROR_REMOTECONN";         break; 			
+	    case CM_ERROR_ATSYS		    : s = "CM_ERROR_ATSYS";              break; 				
+	    case CM_ERROR_NOSUCHPATH	    : s = "CM_ERROR_NOSUCHPATH";         break; 			
+	    case CM_ERROR_CLOCKSKEW	    : s = "CM_ERROR_CLOCKSKEW";          break; 			
+	    case CM_ERROR_BADSMB	    : s = "CM_ERROR_BADSMB";             break; 				
+	    case CM_ERROR_ALLBUSY	    : s = "CM_ERROR_ALLBUSY";            break; 			
+	    case CM_ERROR_NOFILES	    : s = "CM_ERROR_NOFILES";            break; 			
+	    case CM_ERROR_PARTIALWRITE	    : s = "CM_ERROR_PARTIALWRITE";       break; 			
+	    case CM_ERROR_NOIPC		    : s = "CM_ERROR_NOIPC";              break; 				
+	    case CM_ERROR_BADNTFILENAME	    : s = "CM_ERROR_BADNTFILENAME";      break; 			
+	    case CM_ERROR_BUFFERTOOSMALL    : s = "CM_ERROR_BUFFERTOOSMALL";     break; 			
+	    case CM_ERROR_RENAME_IDENTICAL  : s = "CM_ERROR_RENAME_IDENTICAL";   break; 		
+	    case CM_ERROR_ALLOFFLINE        : s = "CM_ERROR_ALLOFFLINE";         break;          
+	    case CM_ERROR_AMBIGUOUS_FILENAME: s = "CM_ERROR_AMBIGUOUS_FILENAME"; break;  
+	    case CM_ERROR_BADLOGONTYPE	    : s = "CM_ERROR_BADLOGONTYPE";       break; 	    
+	    case CM_ERROR_GSSCONTINUE       : s = "CM_ERROR_GSSCONTINUE";        break;         
+	    case CM_ERROR_TIDIPC            : s = "CM_ERROR_TIDIPC";             break;              
+	    case CM_ERROR_TOO_MANY_SYMLINKS : s = "CM_ERROR_TOO_MANY_SYMLINKS";  break;   
+	    case CM_ERROR_PATH_NOT_COVERED  : s = "CM_ERROR_PATH_NOT_COVERED";   break;    
+	    case CM_ERROR_LOCK_CONFLICT     : s = "CM_ERROR_LOCK_CONFLICT";      break;       
+	    case CM_ERROR_SHARING_VIOLATION : s = "CM_ERROR_SHARING_VIOLATION";  break;   
+	    case CM_ERROR_ALLDOWN           : s = "CM_ERROR_ALLDOWN";            break;             
+	    case CM_ERROR_TOOFEWBUFS	    : s = "CM_ERROR_TOOFEWBUFS";         break; 			
+	    case CM_ERROR_TOOMANYBUFS	    : s = "CM_ERROR_TOOMANYBUFS";        break; 			
             }
             osi_Log2(afsd_logp, "cm_Analyze: ignoring error code 0x%x (%s)", 
                      errorCode, s);
@@ -803,17 +838,63 @@ long cm_ConnByServer(cm_server_t *serverp, cm_user_t *userp, cm_conn_t **connpp)
     lock_ReleaseMutex(&userp->mx);
 
     /* return this pointer to our caller */
-    osi_Log1(afsd_logp, "cm_ConnByServer returning conn 0x%x", (long) tcp);
+    osi_Log1(afsd_logp, "cm_ConnByServer returning conn 0x%p", tcp);
     *connpp = tcp;
 
     return 0;
 }
 
+long cm_ServerAvailable(struct cm_fid *fidp, struct cm_user *userp)
+{
+    long code;
+    cm_req_t req;
+    cm_serverRef_t **serverspp;
+    cm_serverRef_t *tsrp;
+    cm_server_t *tsp;
+    int someBusy = 0, someOffline = 0, allOffline = 1, allBusy = 1, allDown = 1;
+
+    cm_InitReq(&req);
+
+    code = cm_GetServerList(fidp, userp, &req, &serverspp);
+    if (code)
+        return 0;
+
+    lock_ObtainWrite(&cm_serverLock);
+    for (tsrp = *serverspp; tsrp; tsrp=tsrp->next) {
+        tsp = tsrp->server;
+        cm_GetServerNoLock(tsp);
+        if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
+	    allDown = 0;
+            if (tsrp->status == busy) {
+		allOffline = 0;
+                someBusy = 1;
+            } else if (tsrp->status == offline) {
+		allBusy = 0;
+		someOffline = 1;
+            } else {
+		allOffline = 0;
+                allBusy = 0;
+            }
+        }
+        cm_PutServerNoLock(tsp);
+    }   
+    lock_ReleaseWrite(&cm_serverLock);
+    cm_FreeServerList(serverspp);
+
+    if (allDown)
+	return 0;
+    else if (allBusy) 
+	return 0;
+    else if (allOffline || (someBusy && someOffline))
+	return 0;
+    else
+	return 1;
+}
+
 long cm_Conn(struct cm_fid *fidp, struct cm_user *userp, cm_req_t *reqp,
              cm_conn_t **connpp)
 {
     long code;
-
     cm_serverRef_t **serverspp;
 
     code = cm_GetServerList(fidp, userp, reqp, &serverspp);
