@@ -16,7 +16,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_flock.c,v 1.29.2.2 2006/06/02 21:25:26 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_flock.c,v 1.29.2.4 2007/02/15 00:35:04 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -26,6 +26,10 @@ RCSID
 #include "afs/afs_osidnlc.h"
 #include "afs/unified_afs.h"
 
+#ifndef OFFSET_MAX
+#define OFFSET_MAX 0x7fffffffffffffff
+#endif
+
 /* Static prototypes */
 static int HandleGetLock(register struct vcache *avc,
 			 register struct AFS_FLOCK *af,
@@ -547,6 +551,10 @@ int afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
 	af->l_len = 0;		/* since some systems indicate it as EOF */
 #endif
 #endif
+    /* Java VMs ask for l_len=(long)-1 regardless of OS/CPU; bottom 32 bits
+     * sometimes get masked off by OS */
+    if (af->l_len == OFFSET_MAX-1)
+	af->l_len = 0;
     /* next line makes byte range locks always succeed,
      * even when they should block */
     if (af->l_whence != 0 || af->l_start != 0 || af->l_len != 0) {
