@@ -14,7 +14,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_segments.c,v 1.16 2003/07/15 23:14:13 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_segments.c,v 1.22 2005/02/12 19:47:03 shadow Exp $");
 
 #include "afs/sysincludes.h"	/*Standard vendor system headers */
 #include "afsincludes.h"	/*AFS-based standard headers */
@@ -47,7 +47,8 @@ afs_StoreMini(register struct vcache *avc, struct vrequest *areq)
     register afs_int32 code;
     register struct rx_call *tcall;
     afs_size_t tlen, xlen = 0;
-    XSTATS_DECLS AFS_STATCNT(afs_StoreMini);
+    XSTATS_DECLS;
+    AFS_STATCNT(afs_StoreMini);
     afs_Trace2(afs_iclSetp, CM_TRACE_STOREMINI, ICL_TYPE_POINTER, avc,
 	       ICL_TYPE_INT32, avc->m.Length);
     tlen = avc->m.Length;
@@ -59,8 +60,8 @@ afs_StoreMini(register struct vcache *avc, struct vrequest *areq)
     do {
 	tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
 	if (tc) {
-	    RX_AFS_GUNLOCK();
 	  retry:
+	    RX_AFS_GUNLOCK();
 	    tcall = rx_NewCall(tc->id);
 	    RX_AFS_GLOCK();
 	    /* Set the client mod time since we always want the file
@@ -103,17 +104,16 @@ afs_StoreMini(register struct vcache *avc, struct vrequest *areq)
 #endif /* AFS_64BIT_CLIENT */
 	    if (code == 0) {
 		code = EndRXAFS_StoreData(tcall, &OutStatus, &tsync);
-#ifdef AFS_64BIT_CLIENT
-		if (code == RXGEN_OPCODE) {
-		    afs_serverSetNo64Bit(tc);
-		    code = rx_EndCall(tcall, code);
-		    goto retry;
-		}
-#endif /* AFS_64BIT_CLIENT */
 	    }
 	    code = rx_EndCall(tcall, code);
 	    RX_AFS_GLOCK();
 	    XSTATS_END_TIME;
+#ifdef AFS_64BIT_CLIENT
+	    if (code == RXGEN_OPCODE && !afs_serverHasNo64Bit(tc)) {
+		afs_serverSetNo64Bit(tc);
+		goto retry;
+	    }
+#endif /* AFS_64BIT_CLIENT */
 	} else
 	    code = -1;
     } while (afs_Analyze
@@ -235,7 +235,6 @@ afs_StoreAllSegments(register struct vcache *avc, struct vrequest *areq,
      */
     origCBs = afs_allCBs;
 
-  retry:
     maxStoredLength = 0;
     tlen = avc->m.Length;
     minj = 0;
@@ -300,12 +299,13 @@ afs_StoreAllSegments(register struct vcache *avc, struct vrequest *areq,
 	    afs_size_t base, bytes;
 	    afs_uint32 nchunks;
 	    int nomore;
-	    unsigned int first;
+	    unsigned int first = 0;
 	    int *shouldwake;
 	    struct conn *tc;
 	    struct osi_file *tfile;
 	    struct rx_call *tcall;
-	    XSTATS_DECLS for (bytes = 0, j = 0; !code && j <= high; j++) {
+	    XSTATS_DECLS;
+	    for (bytes = 0, j = 0; !code && j <= high; j++) {
 		if (dcList[j]) {
 		    ObtainSharedLock(&(dcList[j]->lock), 629);
 		    if (!bytes)
@@ -507,14 +507,8 @@ afs_StoreAllSegments(register struct vcache *avc, struct vrequest *areq,
 						   shouldwake, &lp1, &lp2);
 #endif /* AFS_NOSTATS */
 			    afs_CFileClose(tfile);
-#ifdef AFS_64BIT_CLIENT
-			    if (code == RXGEN_OPCODE) {
-				afs_serverSetNo64Bit(tc);
-				goto restart;
-			    }
-#endif /* AFS_64BIT_CLIENT */
 			    if ((tdc->f.chunkBytes < afs_OtherCSize)
-				&& (i < (nchunks - 1))) {
+				&& (i < (nchunks - 1)) && code == 0) {
 				int bsent, tlen, sbytes =
 				    afs_OtherCSize - tdc->f.chunkBytes;
 				char *tbuffer =
@@ -566,6 +560,12 @@ afs_StoreAllSegments(register struct vcache *avc, struct vrequest *areq,
 			    if (code2)
 				code = code2;
 			}
+#ifdef AFS_64BIT_CLIENT
+			if (code == RXGEN_OPCODE && !afs_serverHasNo64Bit(tc)) {
+			    afs_serverSetNo64Bit(tc);
+			    goto restart;
+			}
+#endif /* AFS_64BIT_CLIENT */
 		    } while (afs_Analyze
 			     (tc, code, &avc->fid, areq,
 			      AFS_STATS_FS_RPCIDX_STOREDATA, SHARED_LOCK,
@@ -896,7 +896,7 @@ afs_InvalidateAllSegments(struct vcache *avc)
 	ObtainWriteLock(&tdc->lock, 679);
 	ZapDCE(tdc);
 	if (vType(avc) == VDIR)
-	    DZap(&tdc->f.inode);
+	    DZap(tdc);
 	ReleaseWriteLock(&tdc->lock);
 	afs_PutDCache(tdc);
     }
@@ -1039,6 +1039,12 @@ afs_TruncateAllSegments(register struct vcache *avc, afs_size_t alen,
 	    afs_CFileTruncate(tfile, newSize);
 	    afs_CFileClose(tfile);
 	    afs_AdjustSize(tdc, newSize);
+	    if (alen < tdc->validPos) {
+                if (alen < AFS_CHUNKTOBASE(tdc->f.chunk))
+                    tdc->validPos = 0;
+                else
+                    tdc->validPos = alen;
+            }
 	    ConvertWToSLock(&tdc->lock);
 	}
 	ReleaseSharedLock(&tdc->lock);
