@@ -15,7 +15,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/bucoord/restore.c,v 1.8.2.1 2004/10/18 07:11:50 shadow Exp $");
+    ("$Header: /cvs/openafs/src/bucoord/restore.c,v 1.10.6.1 2007/01/05 03:32:24 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -23,7 +23,6 @@ RCSID
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #else
-#include <strings.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -44,11 +43,7 @@ extern char *whoami;
 #define	BC_MAXLEVELS	    20
 #define	MAXTAPESATONCE	    10
 
-#ifdef AFS_DEC_ENV
-#define HOSTADDR(sockaddr) (sockaddr)->sin_addr.S_un.S_addr
-#else
 #define HOSTADDR(sockaddr) (sockaddr)->sin_addr.s_addr
-#endif
 
 /* local structure to keep track of volumes and the dumps from which
  * they should be restored 
