@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/update/server.c,v 1.12 2003/12/07 22:49:39 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/update/server.c,v 1.13.14.2 2006/06/20 20:37:12 jaltman Exp $");
 
 #include <afs/stds.h>
 #ifdef	AFS_AIX32_ENV
@@ -50,11 +50,13 @@ RCSID
 #include <afs/cellconfig.h>
 #include <afs/afsutil.h>
 #include <afs/fileutil.h>
+#ifdef	AFS_AIX_ENV
+#include <sys/statfs.h>
+#endif
 #include "update.h"
 #include "global.h"
 
 extern int UPDATE_ExecuteRequest();
-extern rxkad_level StringToLevel(char *name);
 
 static int AddObject(char **expPath, char *dir);
 static int PathInDirectory(char *dir, char *path);
@@ -67,10 +69,14 @@ char *whoami;
 
 static int Quit();
 
+int rxBind = 0;
+
+#define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
+afs_uint32 SHostAddrs[ADDRSPERSITE];
+
 /* check whether caller is authorized to manage RX statistics */
 int
-update_rxstat_userok(call)
-     struct rx_call *call;
+update_rxstat_userok(struct rx_call *call)
 {
     return afsconf_SuperUser(cdir, call, NULL);
 }
@@ -131,11 +137,8 @@ PathInDirectory(char *dir, char *path)
     return inDir;
 }
 
-
 int
-AuthOkay(call, name)
-     struct rx_call *call;
-     char *name;
+AuthOkay(struct rx_call *call, char *name)
 {
     int i;
     rxkad_level level;
@@ -181,12 +184,11 @@ osi_audit()
 #endif
 
 int
-main(argc, argv)
-     int argc;
-     char *argv[];
+main(int argc, char *argv[])
 {
     struct rx_securityClass *securityObjects[3];
     struct rx_service *service;
+    afs_uint32 host = htonl(INADDR_ANY);
 
     int a = 0;
     rxkad_level level;
@@ -238,15 +240,20 @@ main(argc, argv)
 
     for (a = 1; a < argc; a++) {
 	if (argv[a][0] == '-') {	/* parse options */
-	    char arg[256];
-	    lcstring(arg, argv[a], sizeof(arg));
-	    newLevel = StringToLevel(&argv[a][1]);
-	    if (newLevel != -1) {
-		level = newLevel;	/* set new level */
+	    if (strcmp(argv[a], "-rxbind") == 0) {
+		rxBind = 1;
 		continue;
+	    } else {
+		char arg[256];
+		lcstring(arg, argv[a], sizeof(arg));
+		newLevel = rxkad_StringToLevel(&argv[a][1]);
+		if (newLevel != -1) {
+		    level = newLevel;	/* set new level */
+		    continue;
+		}
 	    }
 	  usage:
-	    Quit("Usage: upserver [<directory>+] [-crypt <directory>+] [-clear <directory>+] [-auth <directory>+] [-help]\n");
+	    Quit("Usage: upserver [<directory>+] [-crypt <directory>+] [-clear <directory>+] [-auth <directory>+] [-rxbind] [-help]\n");
 	} else {
 	    int dirlen;
 	    if (nDirs >= sizeof(dirName) / sizeof(dirName[0]))
@@ -274,9 +281,28 @@ main(argc, argv)
 	exit(1);
     }
 
+    if (rxBind) {
+	afs_int32 ccode;
+#ifndef AFS_NT40_ENV
+        if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
+            AFSDIR_SERVER_NETINFO_FILEPATH) {
+            char reason[1024];
+            ccode = parseNetFiles(SHostAddrs, NULL, NULL,
+                                           ADDRSPERSITE, reason,
+                                           AFSDIR_SERVER_NETINFO_FILEPATH,
+                                           AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+        } else 
+#endif	
+	{
+            ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
+        }
+        if (ccode == 1) 
+            host = SHostAddrs[0];
+    }
+
     /* Initialize Rx, telling it port number this server will use for its
      * single service */
-    if (rx_Init(htons(AFSCONF_UPDATEPORT)) < 0)
+    if (rx_InitHost(host, htons(AFSCONF_UPDATEPORT)) < 0)
 	Quit("rx_init");
 
     /* Create a single security object, in this case the null security object,
@@ -300,8 +326,8 @@ main(argc, argv)
      * which is called to decode requests is passed in here
      * (UPDATE_ExecuteRequest). */
     service =
-	rx_NewService(0, UPDATE_SERVICEID, "UPDATE", securityObjects, 3,
-		      UPDATE_ExecuteRequest);
+	rx_NewServiceHost(host, 0, UPDATE_SERVICEID, "UPDATE", securityObjects,
+			  3, UPDATE_ExecuteRequest);
     if (service == (struct rx_service *)0)
 	Quit("rx_NewService");
     rx_SetMaxProcs(service, 2);
@@ -317,9 +343,7 @@ main(argc, argv)
 /* fetch the file name and send it to the remote requester specified by call */
 
 int
-UPDATE_FetchFile(call, name)
-     struct rx_call *call;
-     char *name;
+UPDATE_FetchFile(struct rx_call *call, char *name)
 {
     int fd = -1;
     int error = 0;
@@ -352,9 +376,7 @@ UPDATE_FetchFile(call, name)
 /* fetch dir info about directory name and send it to remote host associated
   with call. */
 int
-UPDATE_FetchInfo(call, name)
-     struct rx_call *call;
-     char *name;
+UPDATE_FetchInfo(struct rx_call *call, char *name)
 {
     int error = 0;
     struct stat status;
@@ -395,19 +417,12 @@ Quit(msg, a, b)
 }
 
 int
-update_SendFile(fd, call, status)
-     register int fd;
-     register struct rx_call *call;
-     register struct stat *status;
+update_SendFile(register int fd, register struct rx_call *call, register struct stat *status)
 {
     char *buffer = (char *)0;
     int blockSize;
     afs_int32 length, tlen;
-#ifdef notdef
-    XDR xdr;
-#endif
 #ifdef	AFS_AIX_ENV
-#include <sys/statfs.h>
     struct statfs tstatfs;
 #endif
 
@@ -427,14 +442,8 @@ update_SendFile(fd, call, status)
 	printf("malloc failed\n");
 	return UPDATE_ERROR;
     }
-#ifdef	notdef
-    xdrrx_create(&xdr, call, XDR_ENCODE);
-    if (!xdr_afs_int32(&xdr, &length))
-	error = UPDATE_ERROR;
-#else
     tlen = htonl(length);
     rx_Write(call, &tlen, sizeof(afs_int32));	/* send length on fetch */
-#endif
     while (!error && length) {
 	register int nbytes = (length > blockSize ? blockSize : length);
 	nbytes = read(fd, buffer, nbytes);
@@ -456,11 +465,10 @@ update_SendFile(fd, call, status)
 /* Enumerate dir (name) and write dir entry info into temp file. 
  */
 int
-update_SendDirInfo(name, call, status, origDir)
-     char *name;		/* Name of dir to enumerate */
-     register struct rx_call *call;	/* rx call */
-     register struct stat *status;	/* stat struct for dir */
-     char *origDir;		/* orig name of dir before being localized */
+update_SendDirInfo(char *name,		/* Name of dir to enumerate */
+     register struct rx_call *call,	/* rx call */
+     register struct stat *status,	/* stat struct for dir */
+     char *origDir)		/* orig name of dir before being localized */
 {
     DIR *dirp;
     struct dirent *dp;
